using Microsoft.AspNetCore.Authentication;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Robware.Auth.API;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Text.Encodings.Web;
using System.Threading.Tasks;

namespace Robware.Api.Auth.Authentication {
	public class ApiKeyAuthenticationHandler : AuthenticationHandler<ApiKeyAuthenticationOptions> {
		private const string ApiKeyHeaderName = "X-Api-Key";
		private readonly IApiKeyValidator _apiKeyValidator;
		private readonly IApiKeys _apiKeys;

		public ApiKeyAuthenticationHandler(
			IOptionsMonitor<ApiKeyAuthenticationOptions> options,
			ILoggerFactory logger,
			UrlEncoder encoder,
			ISystemClock clock,
			IApiKeyValidator apiKeyValidator,
			IApiKeys apiKeys) : base(options, logger, encoder, clock) {
			_apiKeyValidator = apiKeyValidator;
			_apiKeys = apiKeys;
		}

		protected override async Task<AuthenticateResult> HandleAuthenticateAsync() {
			if (!Request.Headers.TryGetValue(ApiKeyHeaderName, out var apiKeyHeaderValues)) {
				return AuthenticateResult.NoResult();
			}

			var providedApiKey = apiKeyHeaderValues.FirstOrDefault();

			if (apiKeyHeaderValues.Count == 0 || string.IsNullOrWhiteSpace(providedApiKey)) {
				return AuthenticateResult.NoResult();
			}

			if (await _apiKeyValidator.Validate(providedApiKey)) {
				var key = await _apiKeys.Get(providedApiKey);
				var claims = new List<Claim>
				{
					new Claim(ClaimTypes.Name, key.Name)
				};

				var identity = new ClaimsIdentity(claims, Options.AuthenticationType);
				var principal = new ClaimsPrincipal(new[] { identity });
				var ticket = new AuthenticationTicket(principal, Options.Scheme);

				return AuthenticateResult.Success(ticket);
			}

			return AuthenticateResult.Fail("Invalid API Key provided.");
		}

		protected override async Task HandleChallengeAsync(AuthenticationProperties properties) {
			Response.StatusCode = 401;
		}

		protected override async Task HandleForbiddenAsync(AuthenticationProperties properties) {
			Response.StatusCode = 403;
		}
	}

}