﻿using System.Threading.Tasks;
using FluentAssertions;
using NSubstitute;
using NSubstitute.ExceptionExtensions;
using Robware.Auth.Users;
using Xunit;

namespace Robware.Auth.Tests.Users {
	public class AuthenticatorTests {
		[Fact]
		public async Task Authenticate_ForUserThatExistsWithCorrectPassword_ReturnsOkResultWithUser() {
			var users = Substitute.For<IUsers>();
			var crypto = Substitute.For<ICryptographyProvider>();
			crypto.Encrypt("password").Returns("password");

			var user = new TestUser("test", "password");
			users.GetByEmail("test").Returns(user);

			var auth = new Authenticator(users, crypto);
			(await auth.Authenticate("test", "password")).Should().BeEquivalentTo((AuthenticationResult.Success, user));
		}

		[Fact]
		public async Task Authenticate_ForUserThatExistsWithIncorrectPassword_ReturnsIncorrectPassword() {
			var users = Substitute.For<IUsers>();
			var crypto = Substitute.For<ICryptographyProvider>();
			crypto.Encrypt("password").Returns("password");

			var user = new TestUser("test", "password");
			users.GetByEmail("test").Returns(user);

			var auth = new Authenticator(users, crypto);

			(await auth.Authenticate("test", "wrong")).Should().BeEquivalentTo((AuthenticationResult.IncorrectPassword, null as User));
		}

		[Fact]
		public async Task Authenticate_ForUserThatDoesntExist_ReturnsNotFound() {
			var users = Substitute.For<IUsers>();
			users.GetByEmail("test").Throws(new UserNotFoundException(""));

			var crypto = Substitute.For<ICryptographyProvider>();
			crypto.Encrypt("password").Returns("password");

			var auth = new Authenticator(users, crypto);

			(await auth.Authenticate("test", "password")).Should().BeEquivalentTo((AuthenticationResult.NotFound, null as User));
		}

		[Fact]
		public async Task UpdateUserPassword_WithExistingUserAndCorrectPassword_ReturnsSuccess() {
			var users = Substitute.For<IUsers>();
			var crypto = Substitute.For<ICryptographyProvider>();
			crypto.Encrypt("correct").Returns("correct");
			crypto.Encrypt("new").Returns("encrypted");

			var user = new TestUser("valid", "correct");
			users.GetByEmail("valid").Returns(user);
			users.UpdateUserPassword("valid", "encrypted").Returns(true);

			var auth = new Authenticator(users, crypto);
			(await auth.UpdateUserPassword("valid", "correct", "new")).Should().Be(AuthenticationResult.Success);
			await users.Received(1).UpdateUserPassword("valid", "encrypted");
		}

		[Fact]
		public async Task UpdateUserPassword_WithExistingUserAndIncorrectPassword_ReturnsIncorrectPassword() {
			var users = Substitute.For<IUsers>();
			var crypto = Substitute.For<ICryptographyProvider>();
			crypto.Encrypt("correct").Returns("correct");

			var user = new TestUser("valid", "correct");
			users.GetByEmail("valid").Returns(user);
			users.UpdateUserPassword("valid", "encrypted").Returns(true);

			var auth = new Authenticator(users, crypto);
			(await auth.UpdateUserPassword("valid", "incorrect", "new")).Should().Be(AuthenticationResult.IncorrectPassword);
			await users.Received(0).UpdateUserPassword("valid", "encrypted");
		}

		[Fact]
		public async Task UpdateUserPassword_WithNonExistentUser_ReturnsNotFound() {
			var users = Substitute.For<IUsers>();
			var crypto = Substitute.For<ICryptographyProvider>();

			users.GetByEmail("invalid").Throws(new UserNotFoundException(""));

			var auth = new Authenticator(users, crypto);
			(await auth.UpdateUserPassword("invalid", "correct", "new")).Should().Be(AuthenticationResult.NotFound);
			await users.Received(0).UpdateUserPassword("valid", "encrypted");
		}
	}
}